package com.ruoyi.system.service.impl;

import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.system.common.ReturnResult;
import com.ruoyi.system.constant.ApiConstant;
import com.ruoyi.system.listener.ExcelListener;
import com.ruoyi.system.mapper.WagesDetailsMapper;
import com.ruoyi.system.mapper.WagesMapper;
import com.ruoyi.system.mapper.WagesSalaryProofMapper;
import com.ruoyi.system.model.doc.CustomDocRequestModel;
import com.ruoyi.system.model.wages.dao.*;
import com.ruoyi.system.model.wages.vo.WagesOverviewOriginalDataVO;
import com.ruoyi.system.service.WagesService;
import com.ruoyi.system.yyinterface.VoucherUtils;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 2024/4/27
 */
@Service
public class WagesServiceImpl implements WagesService {
    private Logger logger = LoggerFactory.getLogger(WagesServiceImpl.class);
    @Autowired
    WagesMapper wagesMapper;
    @Autowired
    WagesSalaryProofMapper wagesSalaryProofMapper;
    @Autowired
    WagesDetailsMapper wagesDetailsMapper;

    @Transactional
    @Override
    public String readOriginalData(MultipartFile file, String date) {
        try {
//            if (Test.put("工资导入", "1")){
//                return "不可重复点击";
//            }
            wagesMapper.deleteOverviewToDate(date);
            wagesMapper.deleteDetailsToDate(date);
            InputStream inputStream1 = file.getInputStream();
            List<WagesDetailsOriginalExcelSheet1DAO> wagesDetSheet1List = EasyExcelFactory.read(inputStream1, WagesDetailsOriginalExcelSheet1DAO.class, new ExcelListener<WagesDetailsOriginalExcelSheet1DAO>()).sheet(0).headRowNumber(1).doReadSync();
            inputStream1.close();
            InputStream inputStream2 = file.getInputStream();
            List<WagesDetailsOriginalExcelSheet2DAO> wagesDetSheet2List = EasyExcelFactory.read(inputStream2, WagesDetailsOriginalExcelSheet2DAO.class, new ExcelListener<WagesDetailsOriginalExcelSheet2DAO>()).sheet(1).headRowNumber(1).doReadSync();
            inputStream2.close();
            InputStream inputStream = file.getInputStream();
            List<WagesOverviewOriginalExcelDAO> wagesOveList = EasyExcelFactory.read(inputStream, WagesOverviewOriginalExcelDAO.class, new ExcelListener<WagesOverviewOriginalExcelDAO>()).sheet(2).headRowNumber(1).doReadSync();
            inputStream.close();
            List<WagesDetailsOriginalExcelDAO> wagesDetList = new ArrayList<>();
            int size = wagesDetSheet1List.size();
            for (int i = 0; i < size; i++) {
                WagesDetailsOriginalExcelSheet1DAO wagesSheet1 = wagesDetSheet1List.get(i);
                WagesDetailsOriginalExcelSheet2DAO wagesSheet2 = wagesDetSheet2List.stream().filter(v -> v.getUnitName().equals(wagesSheet1.getUnitName()))
                        .collect(Collectors.toList()).get(0);
                WagesDetailsOriginalExcelDAO wages = new WagesDetailsOriginalExcelDAO(wagesSheet1.getYear(), wagesSheet1.getMonth(), wagesSheet1.getUnitName(), wagesSheet1.getPeopleCount(), wagesSheet1.getWagesPayable(), wagesSheet1.getHouseRent(), wagesSheet1.getAccommodation(), wagesSheet1.getHeatingFee(), wagesSheet1.getTradeUnionDues(), wagesSheet1.getCompensationDeduction(),
                        wagesSheet1.getOtherDeductions(), wagesSheet1.getMutualMedicalAid(),
                        wagesSheet1.getCommunicationFee(), wagesSheet1.getTax1(), wagesSheet1.getTax2(), wagesSheet1.getPensionInsurance(), wagesSheet1.getUnemploymentInsurance(), wagesSheet1.getMedicalInsurance(), wagesSheet1.getAnnuity(), wagesSheet1.getHousingProvidentFund(),
                        wagesSheet1.getEnterpriseFund(), wagesSheet1.getAnnuityIncomeTax(),
                        wagesSheet2.getTotalWithholding(), wagesSheet2.getNetSalary(),
                        wagesSheet2.getOneChildFee(), wagesSheet2.getMedicalSubsidy(), wagesSheet2.getHealthCareFee(), wagesSheet2.getClassMealFee(), wagesSheet2.getSoapFee(), wagesSheet2.getNursingFee(), wagesSheet2.getHeatstrokePreventionSubsidy(), wagesSheet2.getHeatingSubsidy(), wagesSheet2.getManuscriptFee(), wagesSheet2.getDriverAllowance(), wagesSheet2.getGraduateSettlementFee(), wagesSheet2.getOneTimeDisabilitySubsidy(), wagesSheet2.getLectureAndExamFee(), wagesSheet2.getOtherSubsidies(), wagesSheet2.getTotalPayment(), wagesSheet2.getTotalSalaryDisbursement());
                wagesDetList.add(wages);
            }
            if (wagesDetList.size() > 0) {
                wagesMapper.deleteDetailsOriginal(wagesDetList.get(0).getMonth(), wagesDetList.get(0).getYear());
            }
            if (wagesOveList.size() > 0) {
                wagesMapper.deleteWagesOverviewOriginal(wagesOveList.get(0).getMonth(), wagesOveList.get(0).getYear());
            }
            wagesMapper.insertDetailsOriginal(wagesDetList, date);
            wagesMapper.insertWagesOverviewOriginal(wagesOveList, date);
            return "导入成功！";
        } catch (IOException e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public List<WagesOverviewOriginalDataVO> getSalaryList() {
        logger.info("getSalaryList");
        List<WagesOverviewOriginalDataVO> salaries = wagesMapper.queryList();
        return salaries;
    }

    @Override
    public List<WagesDetailsOriginalExcelDAO> details() {
        return wagesMapper.detailsSelect();
    }

    @Override
    public JSONObject selectWagesVoucherList(String ss) {
        JSONObject object = new JSONObject();
        //List<WagesSalaryProof>  borrowlists = new ArrayList<>();
        JSONArray borrowlists = new JSONArray();
        //List<WagesSalaryProof>  loanlists = new ArrayList<>();
        JSONArray loanlists = new JSONArray();
        JSONObject jsonObject = JSON.parseObject(ss);
        JSONArray borrow = jsonObject.getJSONArray("borrow");
        JSONArray loan = jsonObject.getJSONArray("loan");
        for (int i = 0; i < borrow.size(); i++) {
            JSONObject jsonObject1 = borrow.getJSONObject(i);
            String abStract = jsonObject1.getString("classary");
            String sourceOfExpenses = jsonObject1.getString("name");
            String isDebit = jsonObject1.getString("is_debit");

            WagesSalaryProof wagesSalaryProof = new WagesSalaryProof();
            wagesSalaryProof.setAbStract(abStract);
            wagesSalaryProof.setSourceOfExpenses(sourceOfExpenses);
            wagesSalaryProof.setIsDebit(isDebit);
            WagesSalaryProof wagesSalaryProof1 = wagesSalaryProofMapper.selectWagesVoucherList(wagesSalaryProof);

//            wagesSalaryProof.setAccountCode(wagesSalaryProof1.getAccountCode());
//            wagesSalaryProof.setAuxiliaryItemA(wagesSalaryProof1.getAuxiliaryItemA());
//            wagesSalaryProof.setAuxiliaryItemB(wagesSalaryProof1.getAuxiliaryItemB());
            //borrowlists.add(wagesSalaryProof);
            if (wagesSalaryProof1 != null) {
                System.err.println("wagesSalaryProof1====" + wagesSalaryProof1.toString() + i);
                jsonObject1.put("account_code", wagesSalaryProof1.getAccountCode());
                jsonObject1.put("auxiliary_item_a", wagesSalaryProof1.getAuxiliaryItemA());
                jsonObject1.put("auxiliary_item_b", wagesSalaryProof1.getAuxiliaryItemB());
            } else {
                //如果项目带前带（，去获取相似项目的值
                if (sourceOfExpenses.startsWith("(") || sourceOfExpenses.startsWith("（")) {
                    WagesSalaryProof wagesSalaryProofLike = new WagesSalaryProof();
                    wagesSalaryProofLike.setAbStract(abStract);
                    wagesSalaryProofLike.setSourceOfExpenses("(%");
                    wagesSalaryProofLike.setIsDebit(isDebit);
                    List<WagesSalaryProof> wagesSalaryProofsList = wagesSalaryProofMapper.selectWagesVoucherLikeList(wagesSalaryProofLike);
                    if (wagesSalaryProofsList.size() > 0) {
                        WagesSalaryProof WagesSalaryProof3 = wagesSalaryProofsList.get(0);
                        jsonObject1.put("account_code", WagesSalaryProof3.getAccountCode());
                        jsonObject1.put("auxiliary_item_a", WagesSalaryProof3.getAuxiliaryItemA());
                        int index = sourceOfExpenses.indexOf(")");
                        if (index == -1) {
                            index = sourceOfExpenses.indexOf("）");
                        }
                        if (index >= 0) {
                            String lbname = sourceOfExpenses.substring(index + 1);
                            String lbname2 = "项目";
                            //用友项目接口查询
                            String xmCode = getCodeByName(lbname,"010101");
                            if(xmCode!=null){
                                jsonObject1.put("auxiliary_item_b", lbname2 + ":" + xmCode + "/" + lbname);
                            }

//                            //用友接口取数
//                            CustomDocRequestModel docRequest = new CustomDocRequestModel();
//                            docRequest.setPageIndex(Long.valueOf(1L));
//                            docRequest.setPageSize(Long.valueOf(500L));
//                            docRequest.setName(lbname);
//                            String codeFromCustomDoc = "";
//                            JSONArray customDoc = getCustomDoc(docRequest);
//                            //如果用友返回不为空
//                            if (customDoc != null) {
//                                codeFromCustomDoc = getCodeFromCustomDoc(customDoc, lbname2);
//                                if (codeFromCustomDoc != null) {
//                                    jsonObject1.put("auxiliary_item_b", lbname2 + ":" + codeFromCustomDoc + "/" + lbname);
//                                }
//
//                            }
                        }
                    }


                }
            }

            borrowlists.add(jsonObject1);
        }
        for (int i = 0; i < loan.size(); i++) {
            JSONObject jsonObject2 = loan.getJSONObject(i);
            String abStract = jsonObject2.getString("classary");
            String sourceOfExpenses = jsonObject2.getString("name");
            String isDebit = jsonObject2.getString("is_debit");
            WagesSalaryProof wagesSalaryProof = new WagesSalaryProof();
            wagesSalaryProof.setAbStract(abStract);
            wagesSalaryProof.setSourceOfExpenses(sourceOfExpenses);
            wagesSalaryProof.setIsDebit(isDebit);
            WagesSalaryProof wagesSalaryProof2 = wagesSalaryProofMapper.selectWagesVoucherList(wagesSalaryProof);

//            wagesSalaryProof.setAccountCode(wagesSalaryProof1.getAccountCode());
//            wagesSalaryProof.setAuxiliaryItemA(wagesSalaryProof1.getAuxiliaryItemA());
//            wagesSalaryProof.setAuxiliaryItemB(wagesSalaryProof1.getAuxiliaryItemB());
//            loanlists.add(wagesSalaryProof);
            if (wagesSalaryProof2 != null) {
                System.err.println("wagesSalaryProof2====" + wagesSalaryProof2.toString() + i);
                jsonObject2.put("account_code", wagesSalaryProof2.getAccountCode());
                jsonObject2.put("auxiliary_item_a", wagesSalaryProof2.getAuxiliaryItemA());
                jsonObject2.put("auxiliary_item_b", wagesSalaryProof2.getAuxiliaryItemB());
            }

            loanlists.add(jsonObject2);
        }

//        com.alibaba.fastjson2.JSONArray borrowjson = com.alibaba.fastjson2.JSONArray.of(borrowlists);
//        com.alibaba.fastjson2.JSONArray loanjson = com.alibaba.fastjson2.JSONArray.of(loanlists);
        object.put("borrow", borrowlists);
        object.put("loan", loanlists);
        return object;
    }

    @Override
    public String linXiExcelAnalysisWages(MultipartFile file, String date, String mineId) {
        try {
            List<String> filterList = Arrays.asList("小计", "合计", "总计");
            InputStream inputStream = file.getInputStream();
            List<WagesDetailsExcelDAO> wagesList = EasyExcelFactory.read(inputStream, WagesDetailsExcelDAO.class, new ExcelListener<WagesDetailsExcelDAO>()).sheet(0).headRowNumber(2).doReadSync();
            inputStream.close();
            InputStream inputStream1 = file.getInputStream();
            List<WagesDetailsExcelDAO> financeWagesList = EasyExcelFactory.read(inputStream1, WagesDetailsExcelDAO.class, new ExcelListener<WagesDetailsExcelDAO>()).sheet(1).headRowNumber(2).doReadSync();
            inputStream1.close();
            wagesList = wagesList.parallelStream()
                    .filter(v -> !filterList.contains(v.getA1()))
                    .peek(v -> v.setType("0")).collect(Collectors.toList());
            financeWagesList = financeWagesList.parallelStream()
                    .filter(v -> !filterList.contains(v.getA1()))
                    .peek(v -> v.setType("1")).collect(Collectors.toList());
            wagesDetailsMapper.insertBatch(date, wagesList, mineId);
            wagesDetailsMapper.insertBatch(date, financeWagesList, mineId);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return "完成";
    }

    public JSONArray getCustomDoc(CustomDocRequestModel customDocRequestModel) {
        ReturnResult customDocument = VoucherUtils.callVoucherQuery(customDocRequestModel, "/yonbip/digitalModel/customerdoc/list");
        if (customDocument != null) {
            JSONObject customDocumentJsonObj = (JSONObject) JSON.toJSON(customDocument.getData().get(0));
            return customDocumentJsonObj.getJSONArray("recordList");
        } else {
            return null;
        }

    }

    private String getCodeFromCustomDoc(JSONArray customDoc, String nameZhCn) {
        String code = null;
        for (Object o : customDoc) {
            JSONObject object = (JSONObject) o;
            String custdocdefid_name = object.getString("custdocdefid_name");
            if (custdocdefid_name.equals(nameZhCn)) {
                code = object.getString("code");
                break;
            }
        }
        return code;
    }
    //用友接口获取项目
    private String getCodeByName( String name,String mineId) {
        String code = null;
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("pageIndex",1L);
        jsonObject.put("pageSize",1000L);
        jsonObject.put("name",name);
        ReturnResult accSubject = VoucherUtils.callVoucherQuery( jsonObject , ApiConstant.YFXM_URL);
        JSONObject data = (JSONObject) accSubject.getData().get(0);
        JSONArray recordList= data.getJSONArray("recordList");
        for (Object o : recordList) {
            JSONObject object = (JSONObject)o;
            String orgid_code = object.getString("orgid_code");
            if (orgid_code.equals(mineId)) {
                code = object.getString("code");
                break;
            }
        }
        return code;
    }
}
